import os
import sys
from IPython.testing.globalipapp import start_ipython

from hatchet.external.roundtrip.roundtrip.manager import RoundTrip

no_rt = False
if sys.version_info[0] < 3:
    no_rt = True

local_dir = os.path.dirname(os.path.abspath(__file__))

ip = start_ipython()

if not no_rt:
    RT = RoundTrip(ipy_shell=ip, test=True)


def test_init_load_web_files():
    if not no_rt:
        # verify that the RT Javascript was loaded
        assert (
            open(os.path.join(local_dir, "../roundtrip.js"), "r").read()
            in RT.rt_js._repr_javascript_()
        )

        # test loading multiple files
        RT.load_web_files(
            [
                str(os.path.join(local_dir, "data/test.html")),
                str(os.path.join(local_dir, "data/test.js")),
            ]
        )

        assert len(RT.bridges) != 0

        RT.initialize()

        for b in RT.bridges:
            for s in RT.bridges[b].active_scripts:
                assert (
                    open(os.path.join(local_dir, "data/test.js"), "r").read()
                    in s._repr_javascript_()
                )


def test_data_transfer():
    if not no_rt:
        js_var_down = "data_from_py"
        js_var_up = "data_to_py"

        RT.data_to_js(5, js_variable=js_var_down)
        RT.fetch_data(js_var_up, "retrieved_from_js")

        down = False
        up = False

        for b in RT.bridges:
            for s in RT.bridges[b].active_scripts:
                if (
                    "window.Roundtrip['{}']".format(js_var_down)
                    in s._repr_javascript_()
                ):
                    down = True
                elif "Roundtrip['{}']".format(js_var_up) in s._repr_javascript_():
                    up = True

        assert down is True
        assert up is True


def test_var_transfer():
    if not no_rt:
        js_var_down = "var_from_py"
        js_var_up = "var_to_py"
        py_var = "var_test"

        down = False
        up = False

        # load a var in the user namespace
        ip.run_cell(raw_cell="{} = 1".format(py_var))

        RT.var_to_js(py_var, js_variable=js_var_down)
        RT.fetch_data(js_var_up, "retrieved_from_js")

        for b in RT.bridges:
            for s in RT.bridges[b].active_scripts:
                if (
                    "window.Roundtrip['{}']".format(js_var_down)
                    in s._repr_javascript_()
                    and py_var in s._repr_javascript_()
                ):
                    down = True
                elif "Roundtrip['{}']".format(js_var_up) in s._repr_javascript_():
                    up = True

        assert down is True
        assert up is True



# Copyright 2017-2023 Lawrence Livermore National Security, LLC and other
# Hatchet Project Developers. See the top-level LICENSE file for details.
#
# SPDX-License-Identifier: MIT


class VersionError(Exception):
    """
    Define a version error class with the same features of the base exception class.
    Allows us to catch the very specific ipython version error and throw a warning.
    """

    pass


try:
    from .roundtrip.roundtrip.manager import Roundtrip
    import IPython

    # Refrencing Roundtrip here to resolve scope issues with import
    Roundtrip

    # Testing IPython version
    if int(IPython.__version__.split(".")[0]) > 7:
        raise VersionError()

except ImportError:
    pass

except VersionError:
    print(
        "Warning: Roundtrip module could not be loaded. Requires jupyter notebook version <= 7.x."
    )



from IPython.display import Javascript, HTML, display
from IPython import get_ipython
from bs4 import BeautifulSoup

import io
import tokenize
import json
import inspect
import os
import sys


def _default_converter(data):
    """
    The default converter from an active python datatype to
    a string encoding for transfer to JavaScript.

    :param data: The python varaible/object we are encoding.
    :returns: A string representation of the variable/object which was passed.

    TODO: Raise error if unconvertable.
    """

    if data is None:
        return "{}"
    elif type(data) in [type(""), type(0), type(0.0)]:
        return str(data)
    elif type(data) in [type({}), type([])]:
        return json.dumps(data)
    elif "DataFrame" in str(type(data)) or "Series" in str(type(data)):
        return data.to_json()
    return data


def _default_from_converter(data):
    """
    The default converter from a string representation of JavaScript data.
    Does not do any conversion right now but is required for the javascript code.

    :param data: The string representation of the JavaScript object/data we are decoding.
    :returns: The string representation of the JavaScript object/data passed in.

    """
    return data


class RoundTrip:
    """
    Core Roundtrip class.

    Instantiated as a singleton object "Roundtrip", inside of this file.

    Calls all necessary ipython functions required to load javascript and
    web files. Additionally, it keeps track of python-code specific watched cells
    in the jupyter notebook.

    Most implementation details of automatic running and data binding are deferred
    to the roundtrip object in roundtrip.js.
    """

    def __init__(self, ipy_shell=get_ipython(), test=False):
        """
        Initialize our singelton roundtrip instance.

        :param ipy_shell: The current Jupyter shell.

        TODO: Throw an error if attempting to run outside of a Jupyter notebook.
        """

        if sys.version_info[0] < 3:
            print(
                "Warning: Your Roundtrip visualizations may not load properly. Roundtrip only supports Python v3.x.x. You are using Python v{}.{}.{}".format(
                    sys.version_info[0], sys.version_info[1], sys.version_info[2]
                )
            )

        self.shell = ipy_shell
        self.tags = {
            "script": "<script src='{src}'></script>",
            "style": "<link rel='stylesheet' href='{src}'>",
        }
        self.line = "{tags}\n"
        self.bridges = {}
        self.last_id = None
        self.watched = {}
        self.scrid = 0
        self.relative_html_caches = {}
        self.istest = test

        js_directory = os.path.dirname(os.path.realpath(__file__))

        # Load the javascript roundtrip object upon creation of the python object
        with open(os.path.join(js_directory, "roundtrip.js"), "r") as f:
            script = f.read()

        if test is not True:
            display(Javascript(script))
        else:
            self.rt_js = Javascript(script)
            display(self.rt_js)

    script_map = {"csv": "text/csv", "json": "text/json"}

    def _get_file_type(self, file):
        """
        Get the file extenstion from the filename by retireving the string after the last '.'

        :param file: A string of the format <filename>.<ext>
        """
        return file.split(".")[-1]

    def _file_formatter(self, file):
        ft = self._get_file_type(file)
        if ft in self.script_map.keys():
            tag = self.tags["script"].format(src=file, type=self.script_map[ft])
            return self.line.format(tags=tag)
        if ft == "css":
            tag = self.tags["style"].format(src=file)
            return self.line.format(tags=tag)
        elif ft == "html":
            return open(file).read()

    def load_webpack(self, file, cache=True):
        """
        The primary interface for loading visualizations.

        :param file: A html file containg refrences to all the required javascript required for a visualization to run.
            This html file should be constructed using the HTMLWebpackPlugin with a "public path" defined in the webpack.config.js,
            to simplify the difficulty of routing to find/load local files relative to the location of the visualization extension code.
            See the example_webpack.config.js provided in the roundtrip directory.
        """
        output_html = ""

        # use generic javascript loading for these now
        if cache is False or file not in self.relative_html_caches:
            output_html += self._file_formatter(file)
            html = BeautifulSoup(output_html, "html.parser")
            for tag in html.select("script"):
                # TODO: Add option for server based loading with this
                # So JS can be dynamically loaded
                # tag['src'] = os.path.relpath(tag['src'], self.shell.user_ns['_dh'][0])
                t = tag.extract()
                with open(t["src"]) as f:
                    src = f.read()
                    scrpt = html.new_tag("script")
                    scrpt.string = src
                    html.select("head")[0].append(scrpt)

            output_html = str(html)
            self.relative_html_caches[file] = {"html": output_html}
        else:
            output_html = self.relative_html_caches[file]["html"]

        # This line is needed to expose the current `element` to the webpack bundled scripts as though
        # the scripts were run using display(Javascript()).
        scope_var = """
                    <script id="script-{id}">
                        var element = document.getElementById("script-{id}").parentNode;
                    </script>""".format(
            id=self.scrid
        )
        output_html = scope_var + output_html

        bdg = Bridge(output_html, ipy_shell=self.shell, test=self.istest)

        self.scrid += 1

        self.bridges[bdg.id] = bdg
        self.last_id = bdg.id

        return id

    def load_web_files(self, files):
        """
        The secondary interface for loading visualizations, included for simple visualizations and
        legacy support.

        :param file: A list of filepaths to be loaded into the visualization. These files load
            in-order. So be sure to manage dependencies between scripts correctly by specifying
            dependant scripts after their dependencies.

        """
        output_html = ""

        # this initial string is where variable bindings go
        scripts = [""]

        scope_tag = """
                    <div id="locator-{id}">
                    </div>
            """.format(
            id=self.scrid
        )

        locator_script = (
            'var element = document.getElementById("locator-{id}").parentNode;'.format(
                id=self.scrid
            )
        )

        output_html += scope_tag
        scripts.append(locator_script)

        # load files based on their individual properties
        for file in files:
            ft = self._get_file_type(file)
            if ft == "js":
                scripts.append(open(file).read())
            else:
                output_html += self._file_formatter(file)

        bdg = Bridge(output_html, scripts, self.shell, self.istest)
        self.bridges[bdg.id] = bdg
        self.last_id = bdg.id

        return id

    # Passing to JS is working now
    def data_to_js(
        self,
        data,
        js_variable,
        to_js_converter=_default_converter,
        from_js_converter=_default_from_converter,
    ):
        """
        Pass python data, one-way, into the Javascript Roundtrip oject for retireval in
        loaded visualization/gui code. Intended primarily for passing down data initalized
        within the local scope of a visualization loader.

        :param data: The data to be passed down, can be a variable or literal.
        :param js_variable: String of the key where the data can be found on the window.Roundtrip object in the javascript code.
        :param to_js_converter: Function which defines how to convert the data in 'data' into a serilaizable string format like JSON.
            This function takes one argument "data" and returns a string.
        :param from_js_converter: Function which defines how to convert a string in an specific encoding to an active Python object
            This function takes one argument "data", a string and returns either a python object or the string itself.
        """
        self.bridges[self.last_id].pass_to_js(
            js_variable,
            data,
            py_to_js_converter=to_js_converter,
            js_to_py_converter=from_js_converter,
        )

    # consider seperating watchable and reloadable?
    def var_to_js(
        self,
        jup_var,
        js_variable,
        watch=False,
        to_js_converter=_default_converter,
        from_js_converter=_default_from_converter,
    ):
        """
        Bind a variable in the Jupyter notebook scope to a member of the Roundtrip JS object.

        If 'watch' is True, this binding will propogate data changes to the variable in the Jupyter Notebook
        to the JavaScript code and will also propogate changes to the associated member of the JS Roundtrip object back up to
        the notebook. Changes made under this watch will also cause all cells associated with 'jup_var' and
        annotated with a '?' operator to be automically refreshed and re-run.

        Otherwise it will pass the data in jup_var down one-way like data_to_jS.

        :param jup_var: A string containing the name of a variable in the Jupyter Notebook namespace.
        :param js_variable: String of the key where the data can be found on the window.Roundtrip object in the javascript code.
        :param watch: A boolean which specifies if the two-way binding of the data in jup_var and auto-updating of this cell should be turned on or not.
        :param to_js_converter: Function which defines how to convert the data in 'data' into a serilaizable string format like JSON.
            This function takes one argument "data" and returns a string.
        :param from_js_converter: Function which defines how to convert a string in an specific encoding to an active Python object
            This function takes one argument "data", a string and returns either a python object or the string itself.
        """
        if jup_var[0] == "?":
            if watch is False:
                print(
                    """WARNING:
                        This magic function does not support automatic reloading.
                        Please remove the '?' character in front of '{0}'.
                        """.format(
                        jup_var[1:]
                    )
                )
                watch = "false"
            else:
                watch = "true"

            jup_var = jup_var[1:]

            if (
                jup_var in self.watched.keys()
                and js_variable not in self.watched[jup_var]["js_var"]
            ):
                self.watched[jup_var]["js_var"].append(js_variable)
            else:
                self.watched[jup_var] = {
                    "converter": to_js_converter,
                    "js_var": [js_variable],
                }
        else:
            watch = "false"

        if jup_var not in self.shell.user_ns:
            self.shell.user_ns[jup_var] = None

        data = self.shell.user_ns[jup_var]

        self.bridges[self.last_id].pass_to_js(
            js_variable,
            data,
            two_way=watch,
            python_var=jup_var,
            py_to_js_converter=to_js_converter,
            js_to_py_converter=from_js_converter,
        )

    def manage_jupter_change(self):
        """
        Callback function which runs after a cell is executed.

        Checks to see if a watched variable in a visualization is upaded in the regular python notebook code. Triggers the re-running of all cells
        which have the "?" before the updated variable as an argument to a roundtrip magic function. Also, propogates the data change to the JavaScript code.

        """

        tokens = [
            token
            for token in tokenize.tokenize(
                io.BytesIO(self.shell.user_ns["_ih"][-1].encode("utf-8")).readline
            )
        ]
        assignment_tokens = [
            "=",
            "+=",
            "-=",
            "*=",
            "/=",
            "%=",
            "//=",
            "**=",
            "&=",
            "|=",
            "^=",
            ">>=",
            "<<=",
        ]
        update_flags = {}
        update_hook = """\n (function(){{
                                window.Roundtrip[\'{js_var}\'] = {{
                                    \'data\': \'{data}\',
                                    \'python_var\': \'{var}\',
                                    \'origin\': \'PYASSIGN\'
                                }};
                            }})();\n"""
        code = ""

        for var in self.watched.keys():
            for i, token in enumerate(tokens):
                if token.string == var:
                    lookahead = i
                    while (
                        lookahead < len(tokens)
                        and tokenize.tok_name[tokens[lookahead].type] != "NEWLINE"
                    ):
                        if tokens[lookahead].string in assignment_tokens:
                            update_flags[var] = True
                            break
                        else:
                            lookahead += 1

                    if var in update_flags.keys():
                        break

        for flag in update_flags:
            new_data = self.watched[flag]["converter"](self.shell.user_ns[flag])
            for var in self.watched[flag]["js_var"]:
                code += update_hook.format(js_var=var, data=new_data, var=flag)
        if code != "":
            display(Javascript(code))

    def fetch_data(self, js_var, ipy_var, converter=_default_from_converter):
        """
        Retrieves data from the javascript Roundtrip object.

        :param js_var: String containing the key in the Javascript roundtrip object where the desired data can be found
        :param ipy_var: A String containing the name of a variable in the Jupyter notebook namespace where the retrieved data can be stored
        """
        self.bridges[self.last_id].retrieve_from_js(js_var, ipy_var, converter)

    def initialize(self):
        """
        Function which manages the running and loading of specificed visualization files, data, and variable bindings. Should be called last in
        a visualization magic function.
        """
        self.bridges[self.last_id].run()


class Bridge:
    def __init__(self, html, js=None, ipy_shell=get_ipython(), test=False):
        self.html = html
        self.scripts = js
        self.shell = ipy_shell
        self.display = display(HTML(""), display_id=True)
        self.id = self.display.display_id
        self.converter = _default_converter
        self.istest = test

        if self.istest:
            self.active_scripts = []
            self.active_html = None

    def _extract_simple_dt(self, dt_str):
        return dt_str.split("'")[1]

    def run(self):
        if not self.istest:
            display(HTML(self.html))
        else:
            new_HTML = HTML(self.html)
            self.active_html = new_HTML
            self.display.update(new_HTML)

        if self.scripts is not None:
            js_exe = ""
            for script in self.scripts:
                js_exe += script

            if not self.istest:
                display(Javascript(js_exe))
            else:
                new_Javascript = Javascript(js_exe)
                self.active_scripts.append(new_Javascript)
                display(new_Javascript)

    def add_javascript(self, code):
        if not self.istest:
            display(Javascript(code))
        else:
            new_Javascript = Javascript(code)
            self.active_scripts.append(new_Javascript)
            display(new_Javascript)

    # put down explicit write notification (maybe)
    # watch errors with user documentation
    # run some stress tests on this
    # with weird waits for java script
    # watch gives us an explicit way to link views
    def pass_to_js(
        self,
        js_variable,
        data,
        two_way="false",
        python_var="",
        datatype=None,
        py_to_js_converter=None,
        js_to_py_converter=None,
    ):
        # This may have a race condition; keep an eye on that.
        pass_hook = """\n (function(){{
            window.Roundtrip[\'{js_var}\'] = {{
                \'origin\': \'INIT\',
                \'two_way\': {binding},
                \'python_var\':\'{py_var}\',
                \'type\':\'{type}\',
                \'data\':\'{data}\',
                \'converter\':{converter},
            }};
        }})();\n"""

        if datatype is None:
            datatype = type(data)
            datatype = self._extract_simple_dt(str(datatype))

        try:
            # some default conversion
            # we may want to push this off to the application
            # developer
            if py_to_js_converter is None:
                data = self._default_converter(data)
            else:
                data = py_to_js_converter(data)
                self.converter = py_to_js_converter
        except Exception:
            pass

        # Patch: Ensure all ' and " are escaped
        data = data.replace('"', '\\"').replace("'", "\\'")

        conv_spec = None

        if js_to_py_converter is not None:
            conv_spec = {
                "name": js_to_py_converter.__name__,
                "code": inspect.getsource(js_to_py_converter),
            }

        self.add_javascript(
            pass_hook.format(
                js_var=js_variable,
                binding=two_way,
                py_var=python_var,
                type=datatype,
                data=data,
                converter=json.dumps(conv_spec),
            )
        )

    def retrieve_from_js(self, js_variable, notebook_var, converter):
        # TODO: add validator(s)

        hook_template = """
            (function(){{
                    var holder = Roundtrip['{src}'];
                    holder = `\'${{holder}}\'`;
                    var code = {converter_code};
                    code += `\n{dest} = {converter_name}(${{holder}})`;
                    IPython.notebook.kernel.execute(code, {{
                                                            shell:{{
                                                                reply: function(r){{
                                                                    //consider putting this in a reserved jupyter variable
                                                                    if(r.content.status == \'error\'){{
                                                                        console.error(`${{r.content.ename}} in JS->Python coversion: \n ${{r.content.evalue}}`);
                                                                    }}
                                                                }}
                                                            }}
                                                        }}
                                                    );
                    }})()
               """
        hook = hook_template.format(
            src=str(js_variable),
            dest=str(notebook_var),
            converter_code=json.dumps(inspect.getsource(converter)),
            converter_name=converter.__name__,
        )

        if not self.istest:
            display(Javascript(hook))
        else:
            new_Javascript = Javascript(hook)
            self.active_scripts.append(new_Javascript)
            display(new_Javascript)
        # self.add_javascript(hook)


# Singelton declaration of the roundtrip object.
Roundtrip = RoundTrip()
if Roundtrip.shell is not None:
    Roundtrip.shell.events.register("post_run_cell", Roundtrip.manage_jupter_change)



# Copyright 2017-2023 Lawrence Livermore National Security, LLC and other
# Hatchet Project Developers. See the top-level LICENSE file for details.
#
# SPDX-License-Identifier: MIT

import numpy as np

from hatchet import GraphFrame

import pytest

timemory_avail = True
try:
    import timemory
except ImportError:
    timemory_avail = False


@pytest.mark.skipif(not timemory_avail, reason="timemory package not available")
def test_graphframe(timemory_json_data):
    """Sanity test a GraphFrame object with known data."""
    from timemory.component import WallClock

    wc_s = WallClock.id()  # string identifier
    wc_v = WallClock.index()  # enumeration id
    gf = GraphFrame.from_timemory(timemory_json_data, [wc_s])

    assert len(gf.dataframe) == timemory.size([wc_v])[wc_v]

    for col in gf.dataframe.columns:
        if col in ("sum.wall_clock.inc", "sum.wall_clock"):
            assert gf.dataframe[col].dtype == np.float64
        elif col in ("nid", "rank"):
            assert gf.dataframe[col].dtype == np.int64
        elif col in ("name", "node"):
            assert gf.dataframe[col].dtype == object


@pytest.mark.skipif(not timemory_avail, reason="timemory package not available")
def test_tree(monkeypatch, timemory_json_data):
    """Sanity test a GraphFrame object with known data."""
    monkeypatch.setattr("sys.stdout.isatty", (lambda: False))
    gf = GraphFrame.from_timemory(timemory_json_data)

    output = gf.tree(metric_column="sum.wall_clock")
    print(output)


@pytest.mark.skipif(not timemory_avail, reason="timemory package not available")
def test_graphframe_to_literal(timemory_json_data):
    """Sanity test a GraphFrame object with known data."""
    gf = GraphFrame.from_timemory(timemory_json_data)
    graph_literal = gf.to_literal()

    assert len(graph_literal) == len(gf.graph.roots)


@pytest.mark.skipif(not timemory_avail, reason="timemory package not available")
def test_default_metric(timemory_json_data):
    """Validation test for GraphFrame object using default metric field"""
    gf = GraphFrame.from_timemory(timemory_json_data)

    for func in ["tree", "to_dot", "to_flamegraph"]:
        lhs = "{}".format(getattr(gf, func)(gf.default_metric))
        rhs = "{}".format(getattr(gf, func)())
        assert lhs == rhs



from setuptools import setup
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

# Get the version in a safe way
# per python docs: https://packaging.python.org/guides/single-sourcing-package-version/
version = {}
with open("./roundtrip/version.py") as fp:
    exec(fp.read(), version)


setup(
    name="roundtrip-lib",
    version=version["__version__"],
    description="A Python library for loading JS visualizations into jupyter notebooks.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/hdc-arizona/roundtrip",
    author="Connor Scully-Allison",
    author_email="cscullyallison@email.arizona.edu",
    license="MIT",
    keywords="",
    packages=["roundtrip"],
    install_requires=["numpy", "pandas", "beautifulsoup4", "ipython", "pathlib"],
    include_package_data=True,
)



# Copyright 2017-2023 Lawrence Livermore National Security, LLC and other
# Hatchet Project Developers. See the top-level LICENSE file for details.
#
# SPDX-License-Identifier: MIT

import subprocess
import numpy as np

import pytest
import sys

from hatchet import GraphFrame
from hatchet.readers.caliper_reader import CaliperReader
from hatchet.util.executable import which

caliperreader_avail = True
try:
    import caliperreader
except ImportError:
    caliperreader_avail = False

annotations = [
    "main",
    "LagrangeLeapFrog",
    "LagrangeElements",
    "ApplyMaterialPropertiesForElems",
    "EvalEOSForElems",
    "CalcEnergyForElems",
    "CalcPressureForElems",
    "CalcSoundSpeedForElems",
    "UpdateVolumesForElems",
    "CalcTimeConstraintsForElems",
    "CalcCourantConstraintForElems",
    "CalcHydroConstraintForElems",
    "TimeIncrement",
    "LagrangeNodal",
    "CalcForceForNodes",
    "CalcVolumeForceForElems",
    "IntegrateStressForElems",
    "CalcHourglassControlForElems",
    "CalcFBHourglassForceForElems",
    "CalcLagrangeElements",
    "CalcKinematicsForElems",
    "CalcQForElems",
    "CalcMonotonicQGradientsForElems",
    "CalcMonotonicQRegionForElems",
]


def test_graphframe(lulesh_caliper_json):
    """Sanity test a GraphFrame object with known data."""
    gf = GraphFrame.from_caliper(str(lulesh_caliper_json))

    assert len(gf.dataframe.groupby("name")) == 24

    for col in gf.dataframe.columns:
        if col in ("time (inc)", "time"):
            assert gf.dataframe[col].dtype == np.float64
        elif col in ("nid", "rank"):
            assert gf.dataframe[col].dtype == np.int64
        elif col in ("name", "node"):
            assert gf.dataframe[col].dtype == object

    # TODO: add tests to confirm values in dataframe


def test_read_lulesh_json(lulesh_caliper_json):
    """Sanity check the Caliper reader by examining a known input."""
    reader = CaliperReader(str(lulesh_caliper_json))
    reader.read_json_sections()

    assert len(reader.json_data) == 192
    assert len(reader.json_cols) == 4
    assert len(reader.json_cols_mdata) == 4
    assert len(reader.json_nodes) == 24

    reader.create_graph()
    assert all(an in reader.idx_to_label.values() for an in annotations)


def test_calc_pi_json(calc_pi_caliper_json):
    """Sanity test a GraphFrame object with known data."""
    gf = GraphFrame.from_caliper(str(calc_pi_caliper_json))

    assert len(gf.dataframe.groupby("name")) == 100


@pytest.mark.skipif(not which("cali-query"), reason="needs cali-query to be in path")
def test_lulesh_cali(lulesh_caliper_cali):
    """Sanity check the Caliper reader ingesting a .cali file."""
    grouping_attribute = "function"
    default_metric = "sum(sum#time.duration),inclusive_sum(sum#time.duration)"
    query = "select function,%s group by %s format json-split" % (
        default_metric,
        grouping_attribute,
    )

    gf = GraphFrame.from_caliper(str(lulesh_caliper_cali), query)

    assert len(gf.dataframe.groupby("name")) == 18


@pytest.mark.skipif(not which("cali-query"), reason="needs cali-query to be in path")
def test_lulesh_json_stream(lulesh_caliper_cali):
    """Sanity check the Caliper reader ingesting a JSON string literal."""
    cali_query = which("cali-query")
    grouping_attribute = "function"
    default_metric = "sum(sum#time.duration),inclusive_sum(sum#time.duration)"
    query = "select function,%s group by %s format json-split" % (
        default_metric,
        grouping_attribute,
    )

    cali_json = subprocess.Popen(
        [cali_query, "-q", query, lulesh_caliper_cali], stdout=subprocess.PIPE
    )

    gf = GraphFrame.from_caliper(cali_json.stdout)

    assert len(gf.dataframe.groupby("name")) == 18


@pytest.mark.skipif(sys.version_info > (3, 8), reason="Temporarily allow this to fail.")
def test_filter_squash_unify_caliper_data(lulesh_caliper_json):
    """Sanity test a GraphFrame object with known data."""
    gf1 = GraphFrame.from_caliper(str(lulesh_caliper_json))
    gf2 = GraphFrame.from_caliper(str(lulesh_caliper_json))

    assert gf1.graph is not gf2.graph

    gf1_index_names = gf1.dataframe.index.names
    gf2_index_names = gf2.dataframe.index.names

    gf1.dataframe.reset_index(inplace=True)
    gf2.dataframe.reset_index(inplace=True)

    # indexes are the same since we are reading in the same dataset
    assert all(gf1.dataframe["node"] == gf2.dataframe["node"])

    gf1.dataframe.set_index(gf1_index_names, inplace=True)
    gf2.dataframe.set_index(gf2_index_names, inplace=True)

    squash_gf1 = gf1.filter(lambda x: x["name"].startswith("Calc"))
    squash_gf2 = gf2.filter(lambda x: x["name"].startswith("Calc"))

    squash_gf1.unify(squash_gf2)

    assert squash_gf1.graph is squash_gf2.graph

    squash_gf1.dataframe.reset_index(inplace=True)
    squash_gf2.dataframe.reset_index(inplace=True)

    # Indexes should still be the same after unify. Sort indexes before comparing.
    assert all(squash_gf1.dataframe["node"] == squash_gf2.dataframe["node"])

    squash_gf1.dataframe.set_index(gf1_index_names, inplace=True)
    squash_gf2.dataframe.set_index(gf2_index_names, inplace=True)


def test_tree(monkeypatch, lulesh_caliper_json):
    """Sanity test a GraphFrame object with known data."""
    monkeypatch.setattr("sys.stdout.isatty", (lambda: False))
    gf = GraphFrame.from_caliper(str(lulesh_caliper_json))
    output = gf.tree(metric_column="time")

    assert "121489.000 main" in output
    assert "663.000 LagrangeElements" in output
    assert "21493.000 CalcTimeConstraintsForElems" in output

    output = gf.tree(metric_column="time (inc)")

    assert "662712.000 EvalEOSForElems" in output
    assert "2895319.000 LagrangeNodal" in output


def test_graphframe_to_literal(lulesh_caliper_json):
    """Sanity test a GraphFrame object with known data."""
    gf = GraphFrame.from_caliper(str(lulesh_caliper_json))
    graph_literal = gf.to_literal()

    gf2 = GraphFrame.from_literal(graph_literal)

    assert len(gf.graph) == len(gf2.graph)


def test_graphframe_native_lulesh_from_file(lulesh_caliper_cali):
    """Sanity check the native Caliper reader by examining a known input."""

    gf = GraphFrame.from_caliperreader(str(lulesh_caliper_cali))

    assert len(gf.dataframe.groupby("name")) == 19
    assert "cali.caliper.version" in gf.metadata.keys()

    for col in gf.dataframe.columns:
        if col in ("time (inc)", "time"):
            assert gf.dataframe[col].dtype == np.float64
        elif col in ("nid", "rank"):
            assert gf.dataframe[col].dtype == np.float64
        elif col in ("name", "node"):
            assert gf.dataframe[col].dtype == object

    assert type(gf.metadata["cali.channel"]) == str
    assert type(gf.metadata["cali.caliper.version"]) == str


@pytest.mark.skipif(
    not caliperreader_avail, reason="needs caliper-reader package to be loaded"
)
def test_graphframe_native_lulesh_from_caliperreader(lulesh_caliper_cali):
    """Sanity check the native Caliper reader by examining a known input."""
    r = caliperreader.CaliperReader()
    r.read(lulesh_caliper_cali)

    gf = GraphFrame.from_caliperreader(r)

    assert len(gf.dataframe.groupby("name")) == 19
    assert "cali.caliper.version" in gf.metadata.keys()
    assert type(gf.metadata["cali.caliper.version"]) == str

    for col in gf.dataframe.columns:
        if col in ("time (inc)", "time"):
            assert gf.dataframe[col].dtype == np.float64
        elif col in ("nid", "rank"):
            assert gf.dataframe[col].dtype == np.float64
        elif col in ("name", "node"):
            assert gf.dataframe[col].dtype == object


def test_graphframe_native_lulesh_from_file_node_order(caliper_ordered_cali):
    """Check the order of output from the native Caliper reader by examining a known input with node order column."""

    gf = GraphFrame.from_caliperreader(str(caliper_ordered_cali))

    assert len(gf.dataframe.groupby("name")) == 19
    assert "cali.caliper.version" in gf.metadata.keys()
    assert type(gf.metadata["cali.channel"]) == str
    assert type(gf.metadata["cali.caliper.version"]) == str
    assert "Node order" not in gf.dataframe.columns

    expected_order = [
        "main",
        "lulesh.cycle",
        "TimeIncrement",
        "LagrangeLeapFrog",
        "LagrangeNodal",
        "CalcForceForNodes",
        "CalcVolumeForceForElems",
        "IntegrateStressForElems",
        "CalcHourglassControlForElems",
        "CalcFBHourglassForceForElems",
        "LagrangeElements",
        "CalcLagrangeElements",
        "CalcKinematicsForElems",
        "CalcQForElems",
        "CalcMonotonicQForElems",
        "ApplyMaterialPropertiesForElems",
        "EvalEOSForElems",
        "CalcEnergyForElems",
        "CalcTimeConstraintsForElems",
    ]

    expected_data_order = [
        1.250952,
        1.229935,
        0.000085,
        1.229702,
        0.604766,
        0.566399,
        0.561237,
        0.161196,
        0.395344,
        0.239849,
        0.614079,
        0.175102,
        0.168127,
        0.136318,
        0.038575,
        0.299062,
        0.293046,
        0.190395,
        0.010707,
    ]

    for i in range(0, gf.dataframe.shape[0]):
        # check if the rows are in the expected order
        node_name = gf.dataframe.iloc[i]["name"]
        assert node_name == expected_order[i]
        node_data = gf.dataframe.iloc[i]["Total time"]
        assert node_data == expected_data_order[i]

    # check the tree ordering is correct as well
    output = gf.tree(metric_column="nid")
    for i in range(1, 20):
        location = output.find(str(i))
        assert location != 0
        output = output[location:]


@pytest.mark.skipif(
    not caliperreader_avail, reason="needs caliper-reader package to be loaded"
)
def test_graphframe_native_lulesh_from_caliperreader_node_order(caliper_ordered_cali):
    """Check the order of output from the native Caliper reader by examining a known input with node order column."""
    r = caliperreader.CaliperReader()
    r.read(caliper_ordered_cali)

    gf = GraphFrame.from_caliperreader(r)

    assert len(gf.dataframe.groupby("name")) == 19
    assert "cali.caliper.version" in gf.metadata.keys()
    assert type(gf.metadata["cali.caliper.version"]) == str
    assert "Node order" not in gf.dataframe.columns

    expected_order = [
        "main",
        "lulesh.cycle",
        "TimeIncrement",
        "LagrangeLeapFrog",
        "LagrangeNodal",
        "CalcForceForNodes",
        "CalcVolumeForceForElems",
        "IntegrateStressForElems",
        "CalcHourglassControlForElems",
        "CalcFBHourglassForceForElems",
        "LagrangeElements",
        "CalcLagrangeElements",
        "CalcKinematicsForElems",
        "CalcQForElems",
        "CalcMonotonicQForElems",
        "ApplyMaterialPropertiesForElems",
        "EvalEOSForElems",
        "CalcEnergyForElems",
        "CalcTimeConstraintsForElems",
    ]

    expected_data_order = [
        1.250952,
        1.229935,
        0.000085,
        1.229702,
        0.604766,
        0.566399,
        0.561237,
        0.161196,
        0.395344,
        0.239849,
        0.614079,
        0.175102,
        0.168127,
        0.136318,
        0.038575,
        0.299062,
        0.293046,
        0.190395,
        0.010707,
    ]

    for i in range(0, gf.dataframe.shape[0]):
        # check if the rows are in the expected order
        node_name = gf.dataframe.iloc[i]["name"]
        assert node_name == expected_order[i]
        node_data = gf.dataframe.iloc[i]["Total time"]
        assert node_data == expected_data_order[i]

    # check the tree ordering is correct as well
    output = gf.tree(metric_column="nid")
    for i in range(1, 20):
        location = output.find(str(i))
        assert location != 0
        output = output[location:]


def test_graphframe_lulesh_from_json_node_order(caliper_ordered_json):
    """Check the order of output from the Caliper reader by examining a known json with node order column."""

    gf = GraphFrame.from_caliper(str(caliper_ordered_json))

    assert len(gf.dataframe.groupby("name")) == 19
    assert "cali.caliper.version" in gf.metadata.keys()
    assert type(gf.metadata["cali.channel"]) == str
    assert type(gf.metadata["cali.caliper.version"]) == str
    assert "Node order" not in gf.dataframe.columns

    expected_order = [
        "main",
        "lulesh.cycle",
        "TimeIncrement",
        "LagrangeLeapFrog",
        "LagrangeNodal",
        "CalcForceForNodes",
        "CalcVolumeForceForElems",
        "IntegrateStressForElems",
        "CalcHourglassControlForElems",
        "CalcFBHourglassForceForElems",
        "LagrangeElements",
        "CalcLagrangeElements",
        "CalcKinematicsForElems",
        "CalcQForElems",
        "CalcMonotonicQForElems",
        "ApplyMaterialPropertiesForElems",
        "EvalEOSForElems",
        "CalcEnergyForElems",
        "CalcTimeConstraintsForElems",
    ]

    # check the total time metric to make sure the values are synced with the correct node
    expected_data_order = [
        0.018887,
        0.000154,
        0.000100,
        0.000159,
        0.038220,
        0.005107,
        0.004684,
        0.161500,
        0.163350,
        0.239820,
        0.003603,
        0.007004,
        0.168298,
        0.097482,
        0.038570,
        0.006099,
        0.102499,
        0.190896,
        0.010730,
    ]

    for i in range(0, gf.dataframe.shape[0]):
        # check if the rows are in the expected order
        node_name = gf.dataframe.iloc[i]["name"]
        assert node_name == expected_order[i]
        node_time = gf.dataframe.iloc[i]["time"]
        assert node_time == expected_data_order[i]

    # check the tree ordering is correct as well
    output = gf.tree(metric_column="nid")
    for i in range(1, 20):
        location = output.find(str(i))
        assert location != 0
        output = output[location:]


def test_graphframe_native_lulesh_from_duplicate_node_order(caliper_ordered_dup):
    """Check the order of output from the native Caliper reader by examining a known input with node order column."""

    gf = GraphFrame.from_caliperreader(str(caliper_ordered_dup))

    assert len(gf.dataframe.groupby("name")) == 19
    assert "cali.caliper.version" in gf.metadata.keys()
    assert type(gf.metadata["cali.channel"]) == str
    assert type(gf.metadata["cali.caliper.version"]) == str
    assert "Node order" not in gf.dataframe.columns

    expected_order = [
        "main",
        "lulesh.cycle",
        "TimeIncrement",
        "LagrangeLeapFrog",
        "LagrangeNodal",
        "CalcForceForNodes",
        "CalcVolumeForceForElems",
        "IntegrateStressForElems",
        "CalcHourglassControlForElems",
        "CalcFBHourglassForceForElems",
        "LagrangeElements",
        "CalcLagrangeElements",
        "CalcKinematicsForElems",
        "CalcQForElems",
        "CalcMonotonicQForElems",
        "ApplyMaterialPropertiesForElems",
        "EvalEOSForElems",
        "CalcEnergyForElems",
        "CalcTimeConstraintsForElems",
    ]

    expected_data_order = [
        1.250952,
        1.229935,
        0.000085,
        1.229702,
        0.604766,
        0.566399,
        0.561237,
        0.161196,
        0.395344,
        0.239849,
        0.614079,
        0.175102,
        0.168127,
        0.136318,
        0.038575,
        0.299062,
        0.293046,
        0.190395,
        0.010707,
    ]

    for i in range(0, gf.dataframe.shape[0]):
        # check if the rows are in the expected order
        node_name = gf.dataframe.iloc[i]["name"]
        assert node_name == expected_order[i]
        node_data = gf.dataframe.iloc[i]["Total time"]
        assert node_data == expected_data_order[i]

    # check the tree ordering is correct as well
    output = gf.tree(metric_column="nid")
    for i in range(1, 20):
        location = output.find(str(i))
        assert location != 0
        output = output[location:]


def test_graphframe_lulesh_from_duplicate_json_node_order(caliper_ordered_json_dup):
    """Check the order of output from the Caliper reader by examining a known json with node order column."""

    gf = GraphFrame.from_caliper(str(caliper_ordered_json_dup))

    assert len(gf.dataframe.groupby("name")) == 19
    assert "cali.caliper.version" in gf.metadata.keys()
    assert type(gf.metadata["cali.channel"]) == str
    assert type(gf.metadata["cali.caliper.version"]) == str
    assert "Node order" not in gf.dataframe.columns

    expected_order = [
        "main",
        "lulesh.cycle",
        "TimeIncrement",
        "LagrangeLeapFrog",
        "LagrangeNodal",
        "CalcForceForNodes",
        "CalcVolumeForceForElems",
        "IntegrateStressForElems",
        "CalcHourglassControlForElems",
        "CalcFBHourglassForceForElems",
        "LagrangeElements",
        "CalcLagrangeElements",
        "CalcKinematicsForElems",
        "CalcQForElems",
        "CalcMonotonicQForElems",
        "ApplyMaterialPropertiesForElems",
        "EvalEOSForElems",
        "CalcEnergyForElems",
        "CalcTimeConstraintsForElems",
    ]

    # check the total time metric to make sure the values are synced with the correct node
    expected_data_order = [
        0.018887,
        0.000154,
        0.000100,
        0.000159,
        0.038220,
        0.005107,
        0.004684,
        0.161500,
        0.163350,
        0.239820,
        0.003603,
        0.007004,
        0.168298,
        0.097482,
        0.038570,
        0.006099,
        0.102499,
        0.190896,
        0.010730,
    ]

    for i in range(0, gf.dataframe.shape[0]):
        # check if the rows are in the expected order
        node_name = gf.dataframe.iloc[i]["name"]
        assert node_name == expected_order[i]
        node_time = gf.dataframe.iloc[i]["time"]
        assert node_time == expected_data_order[i]

    # check the tree ordering is correct as well
    output = gf.tree(metric_column="nid")
    for i in range(1, 20):
        location = output.find(str(i))
        assert location != 0
        output = output[location:]


def test_graphframe_native_lulesh_from_file_node_order_mpi(caliper_ordered_cali_mpi):
    """Check the order of output from the native Caliper reader by examining a known input with node order column."""

    gf = GraphFrame.from_caliperreader(str(caliper_ordered_cali_mpi))

    assert len(gf.dataframe.groupby("name")) == 26
    assert "cali.caliper.version" in gf.metadata.keys()
    assert type(gf.metadata["cali.channel"]) == str
    assert type(gf.metadata["cali.caliper.version"]) == str
    assert "Node order" not in gf.dataframe.columns

    expected_order = [
        "main",
        "MPI_Barrier",
        "lulesh.cycle",
        "TimeIncrement",
        "MPI_Allreduce",
        "LagrangeLeapFrog",
        "LagrangeNodal",
        "CalcForceForNodes",
        "CalcVolumeForceForElems",
        "IntegrateStressForElems",
        "CalcHourglassControlForElems",
        "CalcFBHourglassForceForElems",
        "LagrangeElements",
        "CalcLagrangeElements",
        "CalcKinematicsForElems",
        "CalcQForElems",
        "CalcMonotonicQForElems",
        "ApplyMaterialPropertiesForElems",
        "EvalEOSForElems",
        "CalcEnergyForElems",
        "CalcTimeConstraintsForElems",
        "MPI_Reduce",
        "MPI_Finalize",
        "MPI_Initialized",
        "MPI_Finalized",
        "MPI_Comm_dup",
    ]

    expected_data_order = [
        1.264876,
        0.000025,
        1.245757,
        0.000170,
        0.000087,
        1.245438,
        0.607194,
        0.568202,
        0.563011,
        0.161496,
        0.396797,
        0.240649,
        0.627072,
        0.175473,
        0.168321,
        0.136137,
        0.039200,
        0.311724,
        0.305564,
        0.199211,
        0.011015,
        0.000029,
        0.000024,
        0.000007,
        0.000005,
        0.000017,
    ]

    for i in range(0, gf.dataframe.shape[0]):
        # check if the rows are in the expected order
        node_name = gf.dataframe.iloc[i]["name"]
        assert node_name == expected_order[i]
        node_data = gf.dataframe.iloc[i]["Total time"]
        assert node_data == expected_data_order[i]

    # check the tree ordering is correct as well
    output = gf.tree(metric_column="nid")
    for i in range(1, 20):
        location = output.find(str(i))
        assert location != 0
        output = output[location:]


@pytest.mark.skipif(
    not caliperreader_avail, reason="needs caliper-reader package to be loaded"
)
def test_graphframe_native_lulesh_from_caliperreader_node_order_mpi(
    caliper_ordered_cali_mpi,
):
    """Check the order of output from the native Caliper reader by examining a known input with node order column."""
    r = caliperreader.CaliperReader()
    r.read(caliper_ordered_cali_mpi)

    gf = GraphFrame.from_caliperreader(r)

    assert len(gf.dataframe.groupby("name")) == 26
    assert "cali.caliper.version" in gf.metadata.keys()
    assert type(gf.metadata["cali.caliper.version"]) == str
    assert "Node order" not in gf.dataframe.columns

    expected_order = [
        "main",
        "MPI_Barrier",
        "lulesh.cycle",
        "TimeIncrement",
        "MPI_Allreduce",
        "LagrangeLeapFrog",
        "LagrangeNodal",
        "CalcForceForNodes",
        "CalcVolumeForceForElems",
        "IntegrateStressForElems",
        "CalcHourglassControlForElems",
        "CalcFBHourglassForceForElems",
        "LagrangeElements",
        "CalcLagrangeElements",
        "CalcKinematicsForElems",
        "CalcQForElems",
        "CalcMonotonicQForElems",
        "ApplyMaterialPropertiesForElems",
        "EvalEOSForElems",
        "CalcEnergyForElems",
        "CalcTimeConstraintsForElems",
        "MPI_Reduce",
        "MPI_Finalize",
        "MPI_Initialized",
        "MPI_Finalized",
        "MPI_Comm_dup",
    ]

    # check the total time metric to make sure the values are synced with the correct node
    expected_data_order = [
        1.264876,
        0.000025,
        1.245757,
        0.000170,
        0.000087,
        1.245438,
        0.607194,
        0.568202,
        0.563011,
        0.161496,
        0.396797,
        0.240649,
        0.627072,
        0.175473,
        0.168321,
        0.136137,
        0.039200,
        0.311724,
        0.305564,
        0.199211,
        0.011015,
        0.000029,
        0.000024,
        0.000007,
        0.000005,
        0.000017,
    ]

    for i in range(0, gf.dataframe.shape[0]):
        # check if the rows are in the expected order
        node_name = gf.dataframe.iloc[i]["name"]
        assert node_name == expected_order[i]
        node_data = gf.dataframe.iloc[i]["Total time"]
        assert node_data == expected_data_order[i]

    # check the tree ordering is correct as well
    output = gf.tree(metric_column="nid")
    for i in range(1, 20):
        location = output.find(str(i))
        assert location != 0
        output = output[location:]


def test_graphframe_squash_file_node_order(caliper_ordered_cali):
    """Check the order of output from the native Caliper reader by examining a known input with node order column."""

    gf = GraphFrame.from_caliperreader(str(caliper_ordered_cali))

    assert len(gf.dataframe.groupby("name")) == 19
    assert "Node order" not in gf.dataframe.columns

    filtered_gf = gf.filter(lambda x: x["nid"] > 10)
    assert len(filtered_gf.dataframe.groupby("name")) == 9

    expected_order = [
        "LagrangeElements",
        "CalcLagrangeElements",
        "CalcKinematicsForElems",
        "CalcQForElems",
        "CalcMonotonicQForElems",
        "ApplyMaterialPropertiesForElems",
        "EvalEOSForElems",
        "CalcEnergyForElems",
        "CalcTimeConstraintsForElems",
    ]

    expected_data_order = [
        0.614079,
        0.175102,
        0.168127,
        0.136318,
        0.038575,
        0.299062,
        0.293046,
        0.190395,
        0.010707,
    ]

    for i in range(0, filtered_gf.dataframe.shape[0]):
        # check if the rows are in the expected order
        node_name = filtered_gf.dataframe.iloc[i]["name"]
        assert node_name == expected_order[i]
        node_data = filtered_gf.dataframe.iloc[i]["Total time"]
        assert node_data == expected_data_order[i]

    # check the tree ordering is correct as well
    output = filtered_gf.tree(metric_column="nid")
    for i in range(10, 19):
        location = output.find(str(i))
        assert location != 0
        output = output[location:]


def test_inclusive_time_calculation(lulesh_caliper_json):
    """Validate update_inclusive_columns() on known dataset containing per-rank data."""
    gf = GraphFrame.from_caliper(str(lulesh_caliper_json))

    # save original time (inc) column for correctness check
    gf.dataframe["orig_inc_time"] = gf.dataframe["time (inc)"]

    # remove original time (inc) column since it will be generated by update_inclusive_columns()
    del gf.dataframe["time (inc)"]

    gf.update_inclusive_columns()
    assert all(
        gf.dataframe["time (inc)"].values == gf.dataframe["orig_inc_time"].values
    )


def test_sw4_cuda_from_caliperreader(sw4_caliper_cuda_activity_profile_cali):
    gf = GraphFrame.from_caliperreader(sw4_caliper_cuda_activity_profile_cali)

    assert len(gf.graph) == 549
    assert all(
        metric in gf.dataframe.columns for metric in gf.exc_metrics + gf.inc_metrics
    )

    for col in gf.dataframe.columns:
        if col in ("#scale#cupti.activity.duration", "#scale#sum#cupti.host.duration"):
            assert gf.dataframe[col].dtype == np.float64
        elif col in "rank":
            assert gf.dataframe[col].dtype == np.int64
        elif col in "name":
            assert gf.dataframe[col].dtype == object

    for col in gf.exc_metrics + gf.inc_metrics:
        assert col in gf.dataframe.columns

    assert type(gf.metadata["mpi.world.size"]) == int
    assert type(gf.metadata["cali.caliper.version"]) == str
    assert type(gf.metadata["cali.channel"]) == str


def test_sw4_cuda_summary_from_caliperreader(
    sw4_caliper_cuda_activity_profile_summary_cali,
):
    gf = GraphFrame.from_caliperreader(sw4_caliper_cuda_activity_profile_summary_cali)

    assert len(gf.graph) == 393
    assert all(
        metric in gf.dataframe.columns for metric in gf.exc_metrics + gf.inc_metrics
    )

    for col in gf.exc_metrics + gf.inc_metrics:
        assert col in gf.dataframe.columns

    assert type(gf.metadata["mpi.world.size"]) == int
    assert type(gf.metadata["cali.caliper.version"]) == str
    assert type(gf.metadata["cali.channel"]) == str


def test_graphframe_timeseries_lulesh_from_file(caliper_timeseries_cali):
    """Sanity check the timeseries Caliper reader by examining a known input."""

    gf_list = GraphFrame.from_timeseries(str(caliper_timeseries_cali))

    assert (type(gf_list)) == list
    gf = gf_list[0]
    gf2 = gf_list[2]

    assert len(gf.dataframe.groupby("name")) == 19
    assert "cali.caliper.version" in gf.metadata.keys()

    for col in gf.dataframe.columns:
        if col in ("time (inc)", "time"):
            assert gf.dataframe[col].dtype == np.float64
        elif col in ("nid", "rank"):
            assert gf.dataframe[col].dtype == np.float64
        elif col in ("name", "node"):
            assert gf.dataframe[col].dtype == object

    assert type(gf.metadata["cali.channel"]) == str
    assert type(gf.metadata["cali.caliper.version"]) == str

    # check for the expected timeseries and memory allocation columns
    timeseries_cols = [
        "timeseries.starttime",
        "timeseries.duration",
        "loop.start_iteration",
        "loop.iterations",
        "alloc.region.highwatermark",
    ]
    for tcol in timeseries_cols:
        assert tcol in gf.dataframe.columns

    # verify some values are as expected
    assert gf.dataframe["alloc.region.highwatermark"].iloc[0] == 25824351.0
    assert gf.dataframe["alloc.region.highwatermark"].iloc[1] == 63732320.0
    assert np.isnan(gf2.dataframe["loop.start_iteration"].iloc[0])
    assert np.isnan(gf2.dataframe["alloc.region.highwatermark"].iloc[0])



# Copyright 2017-2023 Lawrence Livermore National Security, LLC and other
# Hatchet Project Developers. See the top-level LICENSE file for details.
#
# SPDX-License-Identifier: MIT

import numpy as np
import re

from hatchet import GraphFrame


def test_graphframe(hatchet_cycle_pstats):
    gf = GraphFrame.from_cprofile(str(hatchet_cycle_pstats))

    assert len(gf.dataframe.groupby("file")) == 4
    assert len(gf.dataframe.groupby("name")) == 9

    gf.dataframe.reset_index(inplace=True)

    for col in gf.dataframe.columns:
        if col in ("time (inc)", "time"):
            assert gf.dataframe[col].dtype == np.float64
        elif col in ("line", "numcalls", "nativecalls"):
            assert gf.dataframe[col].dtype == np.int64
        elif col in ("name", "type", "file", "module", "node"):
            assert gf.dataframe[col].dtype == object


def test_tree(monkeypatch, hatchet_cycle_pstats):
    monkeypatch.setattr("sys.stdout.isatty", (lambda: False))
    gf = GraphFrame.from_cprofile(str(hatchet_cycle_pstats))

    output = gf.tree(metric_column="time")

    assert "g pstats_reader_test.py" in output
    assert "<method 'disable' ...Profiler' objects> ~" in output

    output = gf.tree(metric_column="time (inc)")

    assert "f pstats_reader_test.py" in output
    assert re.match("(.|\n)*recursive(.|\n)*recursive", output)
